@@ -7,7 +7,7 @@ This package contains implementations of different factor models:
 
 from .base import BaseFactorModel
 from .dfm import DFM
-from ..config.results import BaseResult, DFMResult, DDFMResult, FitParams
+from ..config import BaseResult, DFMResult, DDFMResult, FitParams
 
 __all__ = [
     'BaseFactorModel', 'DFM',
@@ -16,11 +16,8 @@ __all__ = [
 ]
 
 # DDFM implementation
-from .ddfm import DDFM, DDFMModel
-from .mcmc import DDFMMCMCTrainer
+from .ddfm import DDFM
 __all__.extend([
     'DDFM',  # High-level API
-    'DDFMModel',  # Low-level implementation
-    'DDFMMCMCTrainer',  # MCMC training procedure
 ])
 
