@@ -1,8 +1,7 @@
 """Linear Dynamic Factor Model (DFM) implementation.
 
 This module contains the linear DFM implementation using EM algorithm.
-DFM inherits from BaseFactorModel (not PyTorch Lightning) since all
-calculations are performed in NumPy using pykalman.
+DFM inherits from BaseFactorModel since all calculations are performed in NumPy using pykalman.
 """
 
 # Standard library imports
@@ -42,8 +41,10 @@ from ..config.constants import (
     DEFAULT_DTYPE,
     DEFAULT_CLOCK_FREQUENCY,
     DEFAULT_HIERARCHY_VALUE,
+    DEFAULT_FACTOR_ORDER,
     DEFAULT_IDENTITY_SCALE,
     DEFAULT_ZERO_VALUE,
+    DEFAULT_BLOCK_NAME,
 )
 from ..logger import get_logger
 from .base import BaseFactorModel
@@ -76,7 +77,7 @@ from ..numeric.estimator import (
 )
 
 if TYPE_CHECKING:
-    from ..datamodule import DFMDataModule
+    from ..dataset.dfm_dataset import DFMDataset
 
 _logger = get_logger(__name__)
 
@@ -100,52 +101,7 @@ class DFMTrainingState:
 
 
 class DFM(BaseFactorModel):
-    """High-level API for Linear Dynamic Factor Model.
-    
-    This class implements the EM algorithm for DFM estimation using NumPy and pykalman.
-    It inherits from BaseFactorModel (not PyTorch Lightning) since all calculations
-    are performed in NumPy.
-    
-    **Note**: All calculations are performed in NumPy (using pykalman) for better
-    numerical stability. Parameters are stored as NumPy arrays (no PyTorch dependencies).
-    
-    **Block Structure**: The model supports block-structured factors (factors organized
-    in blocks). Block structure is established during initialization and is preserved
-    during EM updates. pykalman handles the E-step (Kalman filter/smoother), while
-    the M-step uses custom code that maintains block structure, mixed-frequency handling,
-    and idiosyncratic components.
-    
-    Example:
-        >>> from dfm_python import DFM, DFMDataModule
-        >>> from dfm_python.config import DFMConfig
-        >>> import pandas as pd
-        >>> 
-        >>> # Step 1: Load and preprocess data
-        >>> df = pd.read_csv('data/finance.csv')
-        >>> 
-        >>> # Step 2: Create config
-        >>> config = DFMConfig(
-        ...     frequency={'series1': 'm', 'series2': 'm'},
-        ...     blocks={'Block1': {'num_factors': 1, 'series': ['series1', 'series2']}},
-        ...     clock='m'
-        ... )
-        >>> 
-        >>> # Step 3: Create model with config
-        >>> model = DFM(config=config)
-        >>> 
-        >>> # Step 4: Create DataModule and fit
-        >>> dm = DFMDataModule(config=config, data=df, target_series=['series1'])
-        >>> dm.setup()
-        >>> init_params = dm.get_initialization_params()
-        >>> model.fit(X=init_params['X'], datamodule=dm)
-        >>> 
-        >>> # Step 5: Access results
-        >>> result = model.result
-        >>> print(result.summary())
-        >>> 
-        >>> # Step 6: Predict
-        >>> X_forecast, Z_forecast = model.predict(horizon=6)
-    """
+    """Linear Dynamic Factor Model using EM algorithm with NumPy and pykalman."""
     
     def __init__(
         self,
@@ -190,16 +146,22 @@ class DFM(BaseFactorModel):
         """
         super().__init__()
         
-        # Initialize config using consolidated helper method
-        config = self._initialize_config(config)
+        # Initialize config - create temp config if None
+        if config is None:
+            config = self._create_temp_config()
+        self._config = config
         
         # Resolve parameters using consolidated helper
+        # If parameters not explicitly passed, use config values if available, otherwise use defaults
         from ..utils.misc import resolve_param
-        self.threshold = resolve_param(threshold, default=DEFAULT_CONVERGENCE_THRESHOLD)
-        self.max_iter = resolve_param(max_iter, default=DEFAULT_MAX_ITER)
+        # Check config for max_iter and threshold if not explicitly passed
+        config_max_iter = getattr(config, 'max_iter', None) if config is not None else None
+        config_threshold = getattr(config, 'threshold', None) if config is not None else None
+        self.threshold = resolve_param(threshold, default=resolve_param(config_threshold, default=DEFAULT_CONVERGENCE_THRESHOLD))
+        self.max_iter = resolve_param(max_iter, default=resolve_param(config_max_iter, default=DEFAULT_MAX_ITER))
         self.nan_method = resolve_param(nan_method, default=DEFAULT_NAN_METHOD)
         self.nan_k = resolve_param(nan_k, default=DEFAULT_NAN_K)
-        # Mixed frequency: auto-detected from DataModule or config during fit()
+        # Mixed frequency: auto-detected from Dataset or config during fit()
         self._mixed_freq: Optional[bool] = None  # Internal property, auto-detected
         
         # Mixed frequency parameters (set during fit)
@@ -213,7 +175,7 @@ class DFM(BaseFactorModel):
         # Determine number of factors
         # Conditional logic: Initialize num_factors from config if not provided (not validation)
         if num_factors is None:
-            from ..utils.helper import get_config_attr
+            from ..utils.misc import get_config_attr
             factors_per_block = get_config_attr(config, 'factors_per_block', None)
             if factors_per_block is not None:
                 self.num_factors = int(np.sum(factors_per_block))
@@ -232,7 +194,7 @@ class DFM(BaseFactorModel):
             else np.ones(config.get_blocks_array().shape[1]),
             dtype=DEFAULT_DTYPE
         )
-        self.p = 1  # Factors always use AR(1) dynamics (simplified)
+        self.p = DEFAULT_FACTOR_ORDER  # Factors always use AR(1) dynamics (simplified)
         self.blocks = np.array(config.get_blocks_array(), dtype=DEFAULT_DTYPE)
         
         # Parameters stored as NumPy arrays (no PyTorch dependencies)
@@ -250,6 +212,27 @@ class DFM(BaseFactorModel):
         self.data_processed: Optional[np.ndarray] = None
         self.target_scaler: Optional[Any] = None  # Fitted sklearn scaler for target series inverse transformation
     
+    def _create_temp_config(self, block_name: Optional[str] = None) -> DFMConfig:
+        """Create a temporary configuration for model initialization.
+        
+        Parameters
+        ----------
+        block_name : str, optional
+            Name for the default block. If None, uses DEFAULT_BLOCK_NAME.
+            
+        Returns
+        -------
+        DFMConfig
+            Minimal default configuration with a single temporary series and block
+        """
+        if block_name is None:
+            block_name = DEFAULT_BLOCK_NAME
+        
+        return DFMConfig(
+            frequency={'temp': DEFAULT_CLOCK_FREQUENCY},
+            blocks={block_name: {'factors': 1, 'ar_lag': 1, 'clock': 'm'}}
+        )
+    
     def _check_parameters_initialized(self) -> None:
         """Check if model parameters are initialized (required for prediction).
         
@@ -276,14 +259,14 @@ class DFM(BaseFactorModel):
         A, C, Q, R, Z_0, V_0 : np.ndarray
             Parameter arrays
         """
-        from ..utils.common import ensure_numpy
+        import torch
         # Convert to numpy and ensure dtype (only if necessary)
-        self.A = ensure_numpy(A, dtype=DEFAULT_DTYPE) if A is not None else None
-        self.C = ensure_numpy(C, dtype=DEFAULT_DTYPE) if C is not None else None
-        self.Q = ensure_numpy(Q, dtype=DEFAULT_DTYPE) if Q is not None else None
-        self.R = ensure_numpy(R, dtype=DEFAULT_DTYPE) if R is not None else None
-        self.Z_0 = ensure_numpy(Z_0, dtype=DEFAULT_DTYPE) if Z_0 is not None else None
-        self.V_0 = ensure_numpy(V_0, dtype=DEFAULT_DTYPE) if V_0 is not None else None
+        self.A = np.asarray(A, dtype=DEFAULT_DTYPE) if A is not None else None
+        self.C = np.asarray(C, dtype=DEFAULT_DTYPE) if C is not None else None
+        self.Q = np.asarray(Q, dtype=DEFAULT_DTYPE) if Q is not None else None
+        self.R = np.asarray(R, dtype=DEFAULT_DTYPE) if R is not None else None
+        self.Z_0 = np.asarray(Z_0, dtype=DEFAULT_DTYPE) if Z_0 is not None else None
+        self.V_0 = np.asarray(V_0, dtype=DEFAULT_DTYPE) if V_0 is not None else None
     
     def _initialize_clock_freq_idio(
         self,
@@ -676,67 +659,7 @@ class DFM(BaseFactorModel):
         clock: str = DEFAULT_CLOCK_FREQUENCY,
         tent_weights_dict: Optional[Dict[str, np.ndarray]] = None,
     ) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
-        """Initialize DFM state-space parameters (A, C, Q, R, Z_0, V_0).
-        
-        Note: pykalman handles the E-step (Kalman filtering), but we still need to initialize
-        the state-space structure because:
-        1. Block structure must be established (factors organized by blocks)
-        2. Mixed-frequency constraints need tent kernel setup
-        3. Idiosyncratic components require initialization
-        4. Initial parameter estimates are needed for EM algorithm
-        
-        This uses a sequential residual-based PCA approach:
-        1. Handle missing data (spline interpolation)
-        2. Extract factors block-by-block:
-           - Block 1: PCA on original data (after cleaning)
-           - Block 2+: PCA on residuals (after removing previous blocks' contributions)
-        3. Build transition matrices for each block (VAR regression on extracted factors)
-        4. Initialize idiosyncratic components (AR(1) for clock freq, tent chain for slower freq)
-        
-        **Key insight**: The first block extracts factors from original data. Subsequent blocks
-        extract factors from residuals, ensuring each block captures different variance components
-        and factors are orthogonal across blocks.
-        
-        Parameters
-        ----------
-        x : np.ndarray
-            Standardized data matrix (T x N)
-        r : np.ndarray
-            Number of factors per block (n_blocks,)
-        p : int
-            AR lag order (typically 1)
-        blocks : np.ndarray
-            Block structure array (N x n_blocks)
-        opt_nan : dict
-            Missing data handling options {'method': int, 'k': int}
-        R_mat : np.ndarray, optional
-            Constraint matrix for tent kernel aggregation
-        q : np.ndarray, optional
-            Constraint vector for tent kernel aggregation
-        n_slower_freq : int
-            Number of slower-frequency series
-        idio_indicator : np.ndarray, optional
-            Indicator array (1 for clock frequency, 0 for slower frequencies)
-        clock : str
-            Clock frequency ('d', 'w', 'm', 'q', 'sa', 'a')
-        tent_weights_dict : dict, optional
-            Dictionary mapping frequency pairs to tent weights
-            
-        Returns
-        -------
-        A : np.ndarray
-            Initial transition matrix (m x m)
-        C : np.ndarray
-            Initial observation/loading matrix (N x m)
-        Q : np.ndarray
-            Initial process noise covariance (m x m)
-        R : np.ndarray
-            Initial observation noise covariance (N x N)
-        Z_0 : np.ndarray
-            Initial state vector (m,)
-        V_0 : np.ndarray
-            Initial state covariance (m x m)
-        """
+        """Initialize DFM state-space parameters using sequential residual-based PCA."""
         T, N = x.shape
         dtype = DEFAULT_DTYPE
         
@@ -829,7 +752,7 @@ class DFM(BaseFactorModel):
     def fit(
         self,
         X: Union[np.ndarray, Any],
-        datamodule: Optional[Any] = None
+        dataset: Optional[Any] = None
     ) -> DFMTrainingState:
         """Fit model using EM algorithm (wrapper around pykalman).
         
@@ -839,23 +762,24 @@ class DFM(BaseFactorModel):
         Parameters
         ----------
         X : np.ndarray or torch.Tensor, optional
-            Standardized data (T x N). If datamodule is provided, X can be None.
-        datamodule : DFMDataModule, optional
-            Custom DFMDataModule instance. If provided, initialization parameters will be
-            extracted from the datamodule instead of computing them directly.
-            Target scaler is obtained from datamodule.target_scaler for inverse transformation.
+            Standardized data (T x N). If dataset is provided, X can be None.
+        dataset : DFMDataset, optional
+            Custom DFMDataset instance. If provided, initialization parameters will be
+            extracted from the dataset instead of computing them directly.
+            Target scaler is obtained from dataset.target_scaler for inverse transformation.
             
         Returns
         -------
         DFMTrainingState
             Final training state with parameters and convergence info
         """
-        # Use datamodule if provided
-        if datamodule is not None:
-            self._data_module = datamodule  # Store for later use in predict()
+        # Use dataset if provided
+        if dataset is not None:
+            self._dataset = dataset  # Store for later use in predict()
             # Store target_scaler for inverse transformation in predict()
-            self.target_scaler = getattr(datamodule, 'target_scaler', None)
-            init_params = datamodule.get_initialization_params()
+            from ..utils.misc import get_target_scaler
+            self.target_scaler = get_target_scaler(dataset=dataset)
+            init_params = dataset.get_initialization_params()
             X_np = init_params['X']
             R_mat = init_params['R_mat']
             q = init_params['q']
@@ -865,18 +789,18 @@ class DFM(BaseFactorModel):
             idio_indicator = init_params['idio_indicator']
             opt_nan = init_params['opt_nan']
             clock = init_params['clock']
-            # Conditional logic: Auto-detect mixed_freq from DataModule if not set (not validation)
+            # Conditional logic: Auto-detect mixed_freq from Dataset if not set (not validation)
             is_mixed_freq = init_params.get('is_mixed_freq', False)
             if self._mixed_freq is None:
                 self._mixed_freq = is_mixed_freq
         else:
-            # Convert to NumPy using utility function
-            from ..utils.common import ensure_numpy
-            X_np = ensure_numpy(X, dtype=DEFAULT_DTYPE)
+            # Convert to NumPy
+            from ..config.types import to_numpy
+            X_np = to_numpy(X, dtype=DEFAULT_DTYPE)
             
             # Setup mixed-frequency parameters (fallback if no datamodule)
             # Auto-detect from config if not explicitly set
-            from ..utils.helper import get_config_attr
+            from ..utils.misc import get_config_attr
             clock = get_config_attr(self.config, 'clock', DEFAULT_CLOCK_FREQUENCY)
             
             # Conditional logic: Auto-detect mixed frequency from config if not set (not validation)
@@ -951,7 +875,7 @@ class DFM(BaseFactorModel):
             opt_nan = {'method': self.nan_method, 'k': self.nan_k}
         
         # Note: Mx/Wx removed - target_scaler is used instead for inverse transformation
-        # Target scaler is stored in self.target_scaler (set above from datamodule)
+        # Target scaler is stored in self.target_scaler (set above from dataset)
         self.data_processed = X_np
         
         # Rebuild blocks array to match actual data dimensions
@@ -960,9 +884,9 @@ class DFM(BaseFactorModel):
         if self.blocks.shape[0] != N_actual:
             # Rebuild blocks from config using actual data columns
             columns = None
-            if datamodule is not None and hasattr(datamodule, '_processed_columns'):
-                # Use column names from datamodule
-                columns = list(datamodule._processed_columns)
+            if dataset is not None and hasattr(dataset, '_processed_columns'):
+                # Use column names from dataset
+                columns = list(dataset._processed_columns)
             elif hasattr(self._config, 'frequency') and self._config.frequency is not None:
                 # Use series from frequency dict, but filter to match actual data
                 freq_keys = list(self._config.frequency.keys())
@@ -1142,8 +1066,9 @@ class DFM(BaseFactorModel):
         # Compute smoothed data
         x_sm = Z @ C.T
         
-        # Get target scaler from datamodule if available
-        target_scaler = getattr(self, 'target_scaler', None)
+        # Get target scaler from dataset if available
+        from ..utils.misc import get_target_scaler
+        target_scaler = get_target_scaler(model=self)
         
         return DFMResult(
             x_sm=x_sm, Z=Z, C=C, R=R, A=A, Q=Q,
@@ -1213,7 +1138,8 @@ class DFM(BaseFactorModel):
         result = self._ensure_result()
         
         # Get last smoothed state from training as initial state for new data
-        Z_last = result.Z[-1, :] if result.Z.shape[0] > 0 else result.Z_0
+        # Check both shape dimensions and length to safely access last row
+        Z_last = result.Z[-1, :] if (has_shape_with_min_dims(result.Z, min_dims=1) and result.Z.shape[0] > 0) else result.Z_0
         V_last = result.V_0  # Use original V_0 or could compute from last state covariance
         
         # Create Kalman filter with current parameters and new initial state
@@ -1250,7 +1176,7 @@ class DFM(BaseFactorModel):
         """Load configuration from various sources.
         
         After loading config, the model needs to be re-initialized with the new config.
-        For standard Lightning pattern, pass config directly to __init__.
+        For standard pattern, pass config directly to __init__.
         """
         new_config = self._load_config_common(
             source=source,
@@ -1282,10 +1208,10 @@ class DFM(BaseFactorModel):
         """Forecast future values.
         
         This method can be called after training. It uses the training state
-        from the Lightning module to generate forecasts.
+        from the model to generate forecasts.
         
-        Target series are determined from the DataModule's target_series attribute,
-        which should be set during DataModule initialization.
+        Target series are determined from the Dataset's target_series attribute,
+        which should be set during Dataset initialization.
         
         **New Data Initialization**: If `data` is provided, the method runs a
         Kalman filter forward pass on the new data to compute the initial factor
@@ -1324,7 +1250,7 @@ class DFM(BaseFactorModel):
         Raises
         ------
         ValueError
-            If DataModule has no target_series set
+            If Dataset has no target_series set
         ModelNotTrainedError
             If model has not been trained yet
         DataValidationError
@@ -1351,9 +1277,9 @@ class DFM(BaseFactorModel):
             details="This may indicate the model was not properly trained or result object is corrupted"
         )
         
-        # Compute and validate horizon
         if horizon is None:
-            horizon = self._compute_default_horizon()
+            from ..utils.misc import compute_default_horizon
+            horizon = compute_default_horizon(self._config)
         from ..numeric.validator import validate_horizon
         horizon = validate_horizon(horizon)
         
@@ -1377,7 +1303,8 @@ class DFM(BaseFactorModel):
             )
             
             # Get last smoothed state from training as initial state for filtering
-            Z_initial = result.Z[-1, :] if result.Z.shape[0] > 0 else result.Z_0
+            # Check both shape dimensions and length to safely access last row
+            Z_initial = result.Z[-1, :] if (has_shape_with_min_dims(result.Z, min_dims=1) and result.Z.shape[0] > 0) else result.Z_0
             V_initial = result.V_0
             
             # Create Kalman filter with current parameters
@@ -1391,11 +1318,13 @@ class DFM(BaseFactorModel):
             filtered_states, _ = kalman_filter.filter(y_masked)
             
             # Extract last filtered state as initial state for forecasting
-            Z_last = filtered_states[-1, :] if filtered_states.shape[0] > 0 else Z_initial
+            # Check both shape dimensions and length to safely access last row (consistent with other shape checks)
+            Z_last = filtered_states[-1, :] if (has_shape_with_min_dims(filtered_states, min_dims=1) and filtered_states.shape[0] > 0) else Z_initial
         else:
             # Use training state for initial factor state
             # For DFM, we use the last smoothed state from training
-            Z_last = result.Z[-1, :] if result.Z.shape[0] > 0 else np.zeros(result.A.shape[0], dtype=DEFAULT_DTYPE)
+            # Check both shape dimensions and length to safely access last row
+            Z_last = result.Z[-1, :] if (has_shape_with_min_dims(result.Z, min_dims=1) and result.Z.shape[0] > 0) else np.zeros(result.A.shape[0], dtype=DEFAULT_DTYPE)
         
         # Validate factor state and parameters are finite
         from ..numeric.validator import validate_no_nan_inf
@@ -1403,27 +1332,24 @@ class DFM(BaseFactorModel):
         validate_no_nan_inf(A, name="transition matrix A")
         validate_no_nan_inf(C, name="observation matrix C")
         
-        # Resolve target series from DataModule (target_series should be set at initialization)
-        # _resolve_target_series() already validates and raises DataError if none found
+        from ..utils.misc import resolve_target_series
         series_ids = self._config.get_series_ids() if self._config is not None else result.series_ids
-        target_series, target_indices = self._resolve_target_series(series_ids, result)
+        dataset = None
+        try:
+            dataset = self._get_dataset()
+        except (ModelNotInitializedError, AttributeError):
+            pass
+        target_series, target_indices = resolve_target_series(dataset, series_ids, result, self.__class__.__name__)
         
-        # Additional validation: ensure target_series was set in DataModule
+        # Additional validation: ensure target_series was set in Dataset
         if target_series is None or len(target_series) == 0:
             raise ValueError(
-                "DFM prediction failed: no target_series found in DataModule. "
-                "Please set target_series when creating the DataModule (e.g., DFMDataModule(..., target_series=['series_id']))."
+                "DFM prediction failed: no target_series found in Dataset. "
+                "Please set target_series when creating the Dataset (e.g., DFMDataset(..., target_series=['series_id']))."
             )
         
-        # Forecast factors using VAR dynamics (common helper)
-        Z_prev = result.Z[-2, :] if result.Z.shape[0] >= 2 and p == 2 else None
-        Z_forecast = self._forecast_var_factors(
-            Z_last=Z_last,
-            A=A,
-            p=p,
-            horizon=horizon,
-            Z_prev=Z_prev
-        )
+        from ..numeric.estimator import forecast_ar1_factors
+        Z_forecast = forecast_ar1_factors(Z_last, A, horizon, dtype=DEFAULT_DTYPE)
         
         # Optimized: Transform only target series (not all series)
         # Use only target indices for C
@@ -1432,17 +1358,15 @@ class DFM(BaseFactorModel):
         # Transform factors to target observations (in standardized scale)
         X_forecast_std = Z_forecast @ C_target.T  # (horizon x len(target))
         
-        # Unstandardize using scaler if available, otherwise return as-is
+        # Unscale target series using fitted scaler if available
         if target_scaler is not None and hasattr(target_scaler, 'inverse_transform'):
-            # Reshape for scaler: scaler expects (n_samples, n_features)
             X_forecast = target_scaler.inverse_transform(X_forecast_std)
         else:
-            # No scaler - assume already in original scale
             X_forecast = X_forecast_std
         
         # Ensure X_forecast is numpy array and validate it's finite
-        from ..utils.common import ensure_numpy
-        X_forecast = ensure_numpy(X_forecast, dtype=DEFAULT_DTYPE)
+        from ..config.types import to_numpy
+        X_forecast = to_numpy(X_forecast, dtype=DEFAULT_DTYPE)
         validate_no_nan_inf(X_forecast, name="forecast X_forecast")
         
         # Validate forecast values are within reasonable bounds (only if scaler available)
@@ -1450,8 +1374,8 @@ class DFM(BaseFactorModel):
         if target_scaler is not None:
             # Try to extract scale for validation (optional - don't fail if unavailable)
             try:
-                from ..dataset.process import _get_scaler
-                scaler = _get_scaler(target_scaler)
+                # target_scaler is already an instance (fitted in dataset)
+                scaler = target_scaler
                 if scaler is not None and hasattr(scaler, 'scale_'):
                     scale_vals = scaler.scale_
                     if scale_vals is not None and len(scale_vals) > 0:
@@ -1469,8 +1393,9 @@ class DFM(BaseFactorModel):
                                         f"(max deviation: {max_deviation:.1f} std devs). "
                                         f"Possible numerical instability."
                                     )
-            except Exception:
+            except (AttributeError, TypeError, ValueError):
                 # Skip validation if scaler attributes unavailable
+                # Uses specific exception types instead of bare Exception to avoid masking unexpected errors
                 pass
         
         if return_factors:
