@@ -42,7 +42,11 @@ from .stability import (
     stabilize_innovation_covariance,
     create_scaled_identity,
 )
-from ..utils.helper import handle_linear_algebra_error, get_config_attr
+from ..utils.helper import handle_linear_algebra_error
+from ..utils.misc import get_config_attr
+from ..config.constants import DEFAULT_DTYPE
+from .validator import validate_ndarray_ndim, validate_no_nan_inf
+from ..utils.errors import DataValidationError, NumericalError
 
 _logger = get_logger(__name__)
 
@@ -192,6 +196,189 @@ def estimate_var(factors: np.ndarray, order: int = 1) -> Tuple[np.ndarray, np.nd
     return A, Q
 
 
+def get_idio(eps: np.ndarray, idx_no_missings: np.ndarray, min_obs: int = 5) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    """Get AR(1) statistics from eps (matching original TensorFlow get_idio).
+    
+    Returns Phi (AR(1) coefficient matrix), mu_eps (mean), and std_eps (std dev).
+    
+    Parameters
+    ----------
+    eps : np.ndarray
+        Idiosyncratic residuals (T x N)
+    idx_no_missings : np.ndarray
+        Boolean mask (T x N) indicating non-missing values
+    min_obs : int, default 5
+        Minimum number of observations required
+        
+    Returns
+    -------
+    Phi : np.ndarray
+        AR(1) coefficient matrix (N x N), diagonal
+    mu_eps : np.ndarray
+        Mean of idiosyncratic components (N,)
+    std_eps : np.ndarray
+        Standard deviation of idiosyncratic components (N,)
+    """
+    Phi = np.zeros((eps.shape[1], eps.shape[1]))
+    mu_eps = np.zeros(eps.shape[1])
+    std_eps = np.zeros(eps.shape[1])
+    for j in range(eps.shape[1]):
+        to_select = idx_no_missings[:, j]
+        to_select = np.hstack((np.array([False]), to_select[:-1] & to_select[1:]))
+        if np.sum(to_select) >= min_obs:
+            this_eps = eps[to_select, j]
+        else:
+            raise ValueError(f"Not enough observation ({min_obs}) to estimate idio AR(1) parameters.")
+        mu_eps[j] = np.mean(this_eps)
+        std_eps[j] = np.std(this_eps)
+        cov1_eps = np.cov(this_eps[1:], this_eps[:-1])[0][1]
+        Phi[j, j] = cov1_eps / (std_eps[j] ** 2)
+    return Phi, mu_eps, std_eps
+
+
+def get_transition_params(f_t: np.ndarray, eps_t: np.ndarray, bool_no_miss: np.ndarray) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
+    """Calculate transition parameters (matching original TensorFlow get_transition_params).
+    
+    Factor order is fixed to 1 (VAR(1) only).
+    
+    Parameters
+    ----------
+    f_t : np.ndarray
+        Common factors (T x m)
+    eps_t : np.ndarray
+        Idiosyncratic residuals (T x N)
+    bool_no_miss : np.ndarray
+        Boolean mask (T x N) indicating non-missing values
+        
+    Returns
+    -------
+    A : np.ndarray
+        Transition matrix (combines VAR(1) for factors and AR(1) for idiosyncratic)
+    Q : np.ndarray
+        Process noise covariance (W in original)
+    mu_0 : np.ndarray
+        Initial state mean
+    Sigma_0 : np.ndarray
+        Initial state covariance
+    x_t : np.ndarray
+        Latent states [f_t, eps_t]
+    """
+    # Factor order is fixed to 1 (VAR(1) only)
+    f_past = f_t[:-1, :]
+    A_f = (np.linalg.pinv(f_past.T @ f_past) @ f_past.T @ f_t[1:, :]).T
+    
+    Phi, _, _ = get_idio(eps_t, bool_no_miss)
+    
+    x_t = np.vstack((f_t.T, eps_t.T))
+    A = np.vstack((
+        np.hstack((A_f, np.zeros((A_f.shape[0], eps_t.shape[1])))),  # VAR factors
+        np.hstack((np.zeros((eps_t.shape[1], A_f.shape[1])), Phi))  # AR(1) idio
+    ))
+    
+    w_t = x_t[:, 1:] - A @ x_t[:, :-1]
+    Q = np.diag(np.diag(np.cov(w_t)))
+    mu_0 = np.mean(x_t, axis=1)
+    Sigma_0 = np.cov(x_t)
+    Sigma_0[:A_f.shape[1], A_f.shape[1]:] = 0
+    Sigma_0[A_f.shape[1]:, :A_f.shape[1]] = 0
+    Sigma_0[A_f.shape[1]:, A_f.shape[1]:] = np.diag(np.diag(Sigma_0[A_f.shape[1]:, A_f.shape[1]:]))
+    return A, Q, mu_0, Sigma_0, x_t
+
+
+def forecast_ar1_factors(
+    Z_last: np.ndarray,
+    A: np.ndarray,
+    horizon: int,
+    dtype: type = DEFAULT_DTYPE
+) -> np.ndarray:
+    """Forecast factors using AR(1) dynamics.
+    
+    Uses iterative matrix multiplication for efficient computation.
+    
+    **AR(1) Dynamics**: f_t = A @ f_{t-1}
+    
+    This function is used by DFM, DDFM, and KDFM for factor forecasting, ensuring
+    consistent AR dynamics computation across models.
+    
+    Parameters
+    ----------
+    Z_last : np.ndarray
+        Last factor state of shape (m,) where m is number of factors.
+        Must be 1D array with m >= 1.
+    A : np.ndarray
+        Transition matrix of shape (m, m) for AR(1).
+        Must be 2D array with compatible dimensions.
+    horizon : int
+        Number of periods to forecast. Must be >= 1.
+    dtype : type, default=DEFAULT_DTYPE
+        Data type for output array
+        
+    Returns
+    -------
+    np.ndarray
+        Forecasted factors of shape (horizon, m) where:
+        - horizon: Number of forecast periods
+        - m: Number of factors (matches Z_last.shape[0])
+        
+    Raises
+    ------
+    DataValidationError
+        If input shapes are incompatible or invalid
+    NumericalError
+        If matrix operations produce NaN/Inf values
+        If forecast computation produces NaN/Inf values
+    """
+    # Validate inputs
+    validate_ndarray_ndim(Z_last, "Z_last", 1)
+    validate_ndarray_ndim(A, "A", 2)
+    
+    m = Z_last.shape[0]
+    if m < 1:
+        raise DataValidationError(f"Z_last must have at least 1 factor, got m={m}")
+    
+    if horizon < 1:
+        raise DataValidationError(f"horizon must be >= 1, got {horizon}")
+    
+    validate_no_nan_inf(Z_last, name="Z_last")
+    validate_no_nan_inf(A, name="transition matrix A")
+    
+    # Validate A shape
+    if A.shape != (m, m):
+        raise DataValidationError(
+            f"A (transition matrix) must have shape ({m}, {m}), got {A.shape}"
+        )
+    
+    try:
+        # AR(1): f_t = A @ f_{t-1}
+        Z_forecast = np.zeros((horizon, m), dtype=dtype)
+        Z_forecast[0, :] = A @ Z_last
+        for h in range(1, horizon):
+            Z_forecast[h, :] = A @ Z_forecast[h - 1, :]
+        
+        # Validate output
+        validate_no_nan_inf(Z_forecast, name="forecasted factors")
+        
+        if Z_forecast.shape != (horizon, m):
+            raise DataValidationError(
+                f"Forecast output must have shape ({horizon}, {m}), got {Z_forecast.shape}"
+            )
+        
+        return Z_forecast
+        
+    except (RuntimeError, ValueError, TypeError, AttributeError, KeyError, IndexError) as e:
+        # Re-raise configuration/validation errors as-is
+        if isinstance(e, (ConfigurationError, DataValidationError)):
+            raise
+        # Wrap other errors
+        raise NumericalError(
+            f"Forecast computation failed: {e}",
+            details=(
+                f"Z_last shape: {Z_last.shape}, A shape: {A.shape}, "
+                f"horizon={horizon}. Check: (1) Matrix dimensions, "
+                f"(2) Numerical stability, (3) Input validity."
+            )
+        ) from e
+
 
 def estimate_idio_dynamics(
     residuals: np.ndarray,
@@ -221,14 +408,19 @@ def estimate_idio_dynamics(
     Q_eps = np.zeros((N, N))
     
     for j in range(N):
-        # Find valid consecutive pairs (both t-1 and t must be non-missing)
-        valid = ~missing_mask[:, j]
-        valid_pairs = valid[:-1] & valid[1:]
+        # Match original get_idio selection logic:
+        # Original: to_select = np.hstack((np.array([False]), to_select[:-1] * to_select[1:]))
+        # This selects eps[t] where both t-1 and t are non-missing (for t>=1)
+        # Then uses np.cov(this_eps[1:], this_eps[:-1]) to get AR(1) coefficient
+        valid = ~missing_mask[:, j]  # True where non-missing
+        # Select indices where both t-1 and t are non-missing (for t>=1)
+        # First element is False (can't use t=0 without t-1)
+        to_select = np.hstack((np.array([False]), valid[:-1] & valid[1:]))
         
-        if np.sum(valid_pairs) < min_obs:
+        if np.sum(to_select) < min_obs:
             # Insufficient data: use zero AR(1) coefficient
             _logger.warning(
-                f"Insufficient observations ({np.sum(valid_pairs)}) for idio AR(1) "
+                f"Insufficient observations ({np.sum(to_select)}) for idio AR(1) "
                 f"estimation for series {j}. Using zero AR(1) coefficient."
             )
             A_eps[j, j] = DEFAULT_ZERO_VALUE
@@ -238,30 +430,42 @@ def estimate_idio_dynamics(
             else:
                 Q_eps[j, j] = MIN_DIAGONAL_VARIANCE
         else:
-            # Extract valid consecutive pairs
-            eps_t = residuals[1:, j][valid_pairs]
-            eps_t_1 = residuals[:-1, j][valid_pairs]
+            # Extract selected eps values (matches original: this_eps = eps[to_select, j])
+            this_eps = residuals[to_select, j]
             
-            # Estimate AR(1) coefficient using covariance
-            var_eps_t_1 = compute_var_safe(eps_t_1, ddof=0, min_variance=MIN_FACTOR_VARIANCE)
-            if var_eps_t_1 > MIN_FACTOR_VARIANCE:
-                cov_matrix = compute_cov_safe(np.vstack([eps_t, eps_t_1]), rowvar=True, pairwise_complete=False)
+            # Compute AR(1) coefficient using original method:
+            # cov1_eps = np.cov(this_eps[1:], this_eps[:-1])[0][1]
+            # phi = cov1_eps / (std_eps[j] ** 2)
+            # This computes covariance between consecutive pairs in selected sequence
+            if len(this_eps) >= 2:
+                cov_matrix = compute_cov_safe(np.vstack([this_eps[1:], this_eps[:-1]]), rowvar=True, pairwise_complete=False)
                 cov_eps = cov_matrix[0, 1]
-                A_eps[j, j] = cov_eps / var_eps_t_1
-                
-                # Ensure stability: clip AR(1) coefficient
-                if abs(A_eps[j, j]) >= VAR_STABILITY_THRESHOLD:
-                    sign = np.sign(A_eps[j, j])
-                    A_eps[j, j] = sign * VAR_STABILITY_THRESHOLD
-                    _logger.debug(
-                        f"AR(1) coefficient for series {j} clipped to {A_eps[j, j]:.4f} for stability"
-                    )
+                var_eps = compute_var_safe(this_eps[:-1], ddof=0, min_variance=MIN_FACTOR_VARIANCE)
+                if var_eps > MIN_FACTOR_VARIANCE:
+                    A_eps[j, j] = cov_eps / var_eps
+                else:
+                    A_eps[j, j] = DEFAULT_ZERO_VALUE
             else:
                 A_eps[j, j] = DEFAULT_ZERO_VALUE
             
-            # Estimate innovation covariance
-            residuals_ar = eps_t - A_eps[j, j] * eps_t_1
-            Q_eps[j, j] = compute_var_safe(residuals_ar, ddof=0, min_variance=MIN_DIAGONAL_VARIANCE)
+            # Ensure stability: clip AR(1) coefficient
+            if abs(A_eps[j, j]) >= VAR_STABILITY_THRESHOLD:
+                sign = np.sign(A_eps[j, j])
+                A_eps[j, j] = sign * VAR_STABILITY_THRESHOLD
+                _logger.debug(
+                    f"AR(1) coefficient for series {j} clipped to {A_eps[j, j]:.4f} for stability"
+                )
+            
+            # Estimate innovation covariance from AR(1) residuals
+            # Use the same selection as for AR coefficient
+            if len(this_eps) >= 2:
+                eps_t = this_eps[1:]
+                eps_t_1 = this_eps[:-1]
+                residuals_ar = eps_t - A_eps[j, j] * eps_t_1
+                Q_eps[j, j] = compute_var_safe(residuals_ar, ddof=0, min_variance=MIN_DIAGONAL_VARIANCE)
+            else:
+                # Fallback: use variance of selected eps
+                Q_eps[j, j] = compute_var_safe(this_eps, ddof=0, min_variance=MIN_DIAGONAL_VARIANCE)
     
     return A_eps, Q_eps
 
@@ -838,6 +1042,9 @@ __all__ = [
     'estimate_var',
     'estimate_idio_dynamics',
     'estimate_idio_params',
+    # DDFM-specific functions
+    'get_idio',
+    'get_transition_params',
     # Unified estimation functions
     'estimate_var_unified',
     'estimate_ar1_unified',
@@ -845,5 +1052,7 @@ __all__ = [
     'estimate_variance_unified',
     'compute_initial_covariance_from_transition',
     'stabilize_innovation_covariance',
+    # Forecast functions
+    'forecast_ar1_factors',
 ]
 
