@@ -10,9 +10,9 @@ class TestTrainLogger:
         """Test TrainLogger can be initialized."""
         from dfm_python.logger.train_logger import TrainLogger
         
-        logger = TrainLogger(model_name="TestModel", model_type="dfm", verbose=True)
+        logger = TrainLogger(model_name="TestModel", verbose=True)
         assert logger.model_name == "TestModel"
-        assert logger.model_type == "dfm"
         assert logger.verbose is True
-        assert hasattr(logger, 'iterations') or hasattr(logger, 'num_iterations')
+        assert hasattr(logger, 'iterations')
+        assert logger.iterations == 0  # Initialized to 0
 
