@@ -2,10 +2,18 @@
 
 import pytest
 import numpy as np
-from dfm_python.utils.metric import calculate_rmse
 from dfm_python.utils.errors import DataValidationError
 
+# Check if sktime is available
+try:
+    import sktime
+    from dfm_python.utils.metric import calculate_rmse
+    _has_sktime = True
+except ImportError:
+    _has_sktime = False
 
+
+@pytest.mark.skipif(not _has_sktime, reason="sktime is required for calculate_rmse tests")
 class TestCalculateRMSE:
     """Test suite for calculate_rmse."""
     
@@ -15,6 +23,7 @@ class TestCalculateRMSE:
         y_pred = np.array([1.0, 2.0, 3.0])
         rmse_overall, rmse_per_series = calculate_rmse(y_true, y_pred)
         assert rmse_overall == 0.0
+        assert np.all(rmse_per_series == 0.0)
     
     def test_rmse_different_arrays(self):
         """Test RMSE with different arrays."""
@@ -22,6 +31,7 @@ class TestCalculateRMSE:
         y_pred = np.array([2.0, 3.0, 4.0])
         rmse_overall, rmse_per_series = calculate_rmse(y_true, y_pred)
         assert rmse_overall == pytest.approx(1.0)
+        assert np.all(rmse_per_series == pytest.approx(1.0))
     
     def test_rmse_shape_validation(self):
         """Test RMSE with mismatched shapes."""
