@@ -6,6 +6,10 @@ using finance data with market_forward_excess_returns as the target variable.
 Target: market_forward_excess_returns
 Excluded: risk_free_rate, forward_returns
 
+Note: DDFM uses noise injection integrated into the Autoencoder class.
+Noise is pre-sampled on GPU and injected by subtracting epsilon from clean data,
+following the original DDFM pattern: y_t^(mc) = ỹ_t - ε_t^(mc).
+
 """
 
 import sys
@@ -18,12 +22,10 @@ sys.path.insert(0, str(project_root / "src"))
 import pandas as pd
 import numpy as np
 from datetime import datetime
-from dfm_python import DDFM, DDFMDataModule, DDFMTrainer
+from dfm_python import DDFM, DDFMDataset
 from dfm_python.config import DDFMConfig
-from dfm_python.config.constants import TUTORIAL_MAX_PERIODS, DEFAULT_LEARNING_RATE, DEFAULT_BATCH_SIZE, DEFAULT_DDFM_LEARNING_RATE
-from dfm_python.utils.misc import TimeIndex
-from dfm_python.utils.common import select_columns_by_prefix
-from dfm_python.dataset.process import parse_timestamp
+from dfm_python.config.constants import TUTORIAL_MAX_PERIODS, DEFAULT_LEARNING_RATE, DEFAULT_DDFM_WINDOW_SIZE, DEFAULT_DDFM_LEARNING_RATE
+from dfm_python.utils.misc import select_columns_by_prefix
 from sklearn.pipeline import Pipeline
 from sklearn.preprocessing import StandardScaler
 from sktime.transformations.series.impute import Imputer
@@ -136,14 +138,28 @@ missing_after = df_preprocessed.isnull().sum().sum()
 print(f"   Missing values after preprocessing: {missing_after}")
 print(f"   Preprocessed data shape: {df_preprocessed.shape}")
 
+# Standardize ALL data (matching original TensorFlow DDFM)
+# Original TensorFlow: self.data = (data - self.mean_z) / self.sigma_z
+print("\n[Step 2.6] Standardizing ALL data (matching original TensorFlow DDFM)...")
+print("   Original TensorFlow: self.data = (data - self.mean_z) / self.sigma_z")
+print("   All series (including target) must be standardized before passing to Dataset")
+
+# Standardize all data (matching original TensorFlow DDFM)
+mean_z = df_preprocessed.mean().values
+sigma_z = df_preprocessed.std().values
+df_standardized = (df_preprocessed - mean_z) / sigma_z
+
 # Verify standardization
-mean_vals = df_preprocessed.mean()
-std_vals = df_preprocessed.std()
+mean_vals = df_standardized.mean()
+std_vals = df_standardized.std()
 max_mean = float(mean_vals.abs().max())
 max_std_dev = float((std_vals - 1.0).abs().max())
 print(f"   Standardization check - Max |mean|: {max_mean:.6f} (should be ~0)")
 print(f"   Standardization check - Max |std - 1|: {max_std_dev:.6f} (should be ~0)")
 
+# Update df_preprocessed to use standardized data
+df_preprocessed = df_standardized
+
 # Update df_processed to use preprocessed data
 df_processed = df_preprocessed
 
@@ -158,86 +174,92 @@ frequency_dict = {col: "m" for col in selected_cols}
 
 # Create DDFM config (DDFM does not use blocks structure)
 # DDFM uses num_factors directly, not blocks
+# Note: factor_order is not a parameter - factors always use AR(1) dynamics
 config = DDFMConfig(
     frequency=frequency_dict,
     clock="m",  # Monthly clock frequency
     num_factors=1,  # Reduced to 1 for faster execution
-    factor_order=1,  # VAR(1) - first-order autoregressive
     encoder_layers=[32, 16],  # Reduced for faster execution
-    epochs=10,  # Reduced for faster execution
+    n_mc_samples=10,  # Number of MC samples per MCMC iteration (reduced for faster execution)
     learning_rate=DEFAULT_LEARNING_RATE,
-    batch_size=DEFAULT_BATCH_SIZE,
+    window_size=DEFAULT_DDFM_WINDOW_SIZE,  # Window size (time-step batch size) for training
     target_scaler=y_scaler  # Fitted scaler for target series inverse transformation
 )
 
 print(f"   Number of series: {len(selected_cols)}")
 print(f"   Number of factors: {config.num_factors} (DDFM uses num_factors parameter)")
-print(f"   Factor dynamics: VAR(1) (factor_order=1)")
+print(f"   Factor dynamics: VAR(1) (always AR(1), not configurable)")
+print(f"   MC samples per iteration: {config.n_mc_samples}")
 print(f"   Target series: {target_col}")
+print(f"   Noise injection: Integrated in Autoencoder (pre-sampled on GPU)")
 
 # ============================================================================
-# Step 4: Create DataModule
+# Step 4: Create Dataset
 # ============================================================================
-print("\n[Step 4] Creating DataModule...")
+print("\n[Step 4] Creating Dataset...")
 
-# Create time index (assuming monthly data)
-# For finance data, date_id is an index, so we'll create a simple time index
-# Use a recent start date to avoid overflow
-n_periods = len(df_processed)
-# Start from 1980 to ensure we don't hit overflow
-start_date = datetime(1980, 1, 1)
-time_list = [
-    (pd.Timestamp(start_date) + pd.DateOffset(months=i)).to_pydatetime()
-    for i in range(n_periods)
-]
+# Note: DDFMDataset uses time_idx='index' to use DataFrame index as time identifier
+# No need to create separate TimeIndex object
 
-time_index = TimeIndex(time_list)
-
-# Create DDFMDataModule with preprocessed data
-# Data must be preprocessed before passing to DataModule
+# Create DDFMDataset with preprocessed data
+# Data must be preprocessed before passing to Dataset
 # Target series are specified separately - they remain in raw form (not preprocessed)
-data_module = DDFMDataModule(
-    config=config,
+dataset = DDFMDataset(
     data=df_processed,  # Pass DataFrame directly (not .values) - already preprocessed
-    time_index=time_index,
-    target_series=[target_col]  # Specify target series
+    time_idx='index',  # Use DataFrame index as time identifier
+    target_series=[target_col],  # Specify target series
+    target_scaler=y_scaler  # Fitted scaler for target series
 )
-data_module.setup()
 
-print(f"   DataModule created successfully")
-if hasattr(data_module, 'data_processed') and data_module.data_processed is not None:
-    print(f"   Processed data shape: {data_module.data_processed.shape}")
-else:
-    print(f"   Data shape: {df_processed.shape}")
+print(f"   Dataset created successfully")
+print(f"   Data shape: {dataset.data.shape}")
+print(f"   Target series: {dataset.target_series}")
 
 # ============================================================================
 # Step 5: Train Model
 # ============================================================================
 print("\n[Step 5] Training DDFM model...")
 
+# Create model with parameters from config
+# Map config parameters to model parameters
+encoder_layers = getattr(config, 'encoder_layers', [32, 1])
+encoder_size = tuple(encoder_layers) if encoder_layers else (32, 1)
+max_epoch = getattr(config, 'max_epoch', 3)  # Reduced for faster execution
+
 model = DDFM(
-    encoder_layers=[32, 16],  # Reduced for faster execution
-    num_factors=1,  # Reduced to 1 for faster execution
-    factor_order=1,  # VAR(1) - first-order factor dynamics
-    epochs=10,  # Reduced for faster execution
-    max_iter=3,  # Reduced for faster execution
-    batch_size=DEFAULT_BATCH_SIZE,  # Reduced for faster execution
-    learning_rate=DEFAULT_DDFM_LEARNING_RATE
+    dataset=dataset,
+    config=config,
+    encoder_size=encoder_size,  # Convert list to tuple
+    decoder_type="linear",
+    activation=getattr(config, 'activation', 'relu'),
+    learning_rate=getattr(config, 'learning_rate', DEFAULT_DDFM_LEARNING_RATE),
+    optimizer='Adam',
+    n_mc_samples=getattr(config, 'n_mc_samples', 10),
+    window_size=getattr(config, 'window_size', DEFAULT_DDFM_WINDOW_SIZE),
+    max_iter=max_epoch,  # Map max_epoch -> max_iter
+    tolerance=getattr(config, 'tolerance', 0.0005),
+    disp=getattr(config, 'disp', 10),
+    seed=getattr(config, 'seed', None)
 )
-model._config = config  # Set config directly
-
-trainer = DDFMTrainer(max_epochs=1)  # Minimal epochs for faster execution
-trainer.fit(model, data_module)
 
+# Train model - fit() builds model, pre-trains, and trains in one method
+print(f"   Starting training (max {model.max_iter} iterations)...")
+model.fit()
 print("   Training completed!")
 
+# Build state-space model for prediction
+print("\n[Step 5.5] Building state-space model...")
+model.build_state_space()
+print("   State-space model built successfully")
+
 # ============================================================================
 # Step 6: Prediction
 # ============================================================================
 print("\n[Step 6] Making predictions...")
 
-# Predict with horizon=6 (uses target_series from DataModule)
-X_forecast, Z_forecast = model.predict(horizon=6)
+# Predict with horizon=6 (uses target_series from Dataset)
+# Note: predict() requires build_state_space() to be called first
+X_forecast, Z_forecast = model.predict(horizon=6, return_series=True, return_factors=True)
 
 print(f"   Forecast shape: {X_forecast.shape}")
 print(f"   Factor forecast shape: {Z_forecast.shape}")
