@@ -5,6 +5,10 @@ using macro data with KOEQUIPTE as the target variable.
 
 Target: KOEQUIPTE (Investment, Equipment, Estimation, SA)
 
+Note: DDFM uses noise injection integrated into the Autoencoder class.
+Noise is pre-sampled on GPU and injected by subtracting epsilon from clean data,
+following the original DDFM pattern: y_t^(mc) = ỹ_t - ε_t^(mc).
+
 """
 
 import sys
@@ -17,10 +21,9 @@ sys.path.insert(0, str(project_root / "src"))
 import pandas as pd
 import numpy as np
 from datetime import datetime
-from dfm_python import DDFM, DDFMDataModule, DDFMTrainer
+from dfm_python import DDFM, DDFMDataset
 from dfm_python.config import DDFMConfig
-from dfm_python.config.constants import TUTORIAL_MAX_PERIODS, DEFAULT_LEARNING_RATE, DEFAULT_BATCH_SIZE, DEFAULT_DDFM_LEARNING_RATE
-from dfm_python.utils.misc import TimeIndex
+from dfm_python.config.constants import TUTORIAL_MAX_PERIODS, DEFAULT_LEARNING_RATE, DEFAULT_DDFM_WINDOW_SIZE, DEFAULT_DDFM_LEARNING_RATE
 from sklearn.pipeline import Pipeline
 from sklearn.preprocessing import StandardScaler
 from sktime.transformations.series.impute import Imputer
@@ -157,7 +160,7 @@ if isinstance(df_preprocessed, np.ndarray):
 elif not isinstance(df_preprocessed, pd.DataFrame):
     df_preprocessed = pd.DataFrame(df_preprocessed)
 
-# Add date column back for DataModule to extract
+# Add date column back for Dataset to extract
 # Date column was separated earlier, so add it back now
 df_preprocessed['date'] = date_column.values
 
@@ -194,76 +197,92 @@ frequency_dict = {col: "m" for col in selected_cols}
 
 # Create DDFM config (DDFM does not use blocks structure)
 # DDFM uses num_factors directly, not blocks
+# Note: factor_order is not a parameter - factors always use AR(1) dynamics
 config = DDFMConfig(
     frequency=frequency_dict,
     clock="m",  # Monthly clock frequency
     num_factors=1,  # Reduced to 1 for faster execution
-    factor_order=1,  # VAR(1) - first-order autoregressive
     encoder_layers=[32, 16],  # Reduced for faster execution
-    epochs=10,  # Reduced for faster execution
+    n_mc_samples=10,  # Number of MC samples per MCMC iteration (reduced for faster execution)
     learning_rate=DEFAULT_LEARNING_RATE,
-    batch_size=DEFAULT_BATCH_SIZE,
+    window_size=DEFAULT_DDFM_WINDOW_SIZE,  # Window size (time-step batch size) for training
     target_scaler=y_scaler  # Fitted scaler for target series inverse transformation
 )
 
 print(f"   Number of series: {len(selected_cols)}")
 print(f"   Number of factors: {config.num_factors} (DDFM uses num_factors parameter)")
-print(f"   Factor dynamics: VAR(1) (factor_order=1)")
+print(f"   Factor dynamics: VAR(1) (always AR(1), not configurable)")
+print(f"   MC samples per iteration: {config.n_mc_samples}")
 print(f"   Target series: {target_col}")
+print(f"   Noise injection: Integrated in Autoencoder (pre-sampled on GPU)")
 
 # ============================================================================
-# Step 4: Create DataModule
+# Step 4: Create Dataset
 # ============================================================================
-print("\n[Step 4] Creating DataModule...")
+print("\n[Step 4] Creating Dataset...")
 
-# Create DDFMDataModule with preprocessed data
-# Data must be preprocessed before passing to DataModule
+# Create DDFMDataset with preprocessed data
+# Data must be preprocessed before passing to Dataset
 # Target series are specified separately - they remain in raw form (not preprocessed)
-# time_index_column='date' will extract time index from DataFrame and remove the column
-data_module = DDFMDataModule(
-    config=config,
-    data=df_processed,  # Pass DataFrame directly (not .values) - already preprocessed
-    time_index_column='date',  # Extract time index from 'date' column and exclude it from data
-    target_series=[target_col]  # Specify target series
-)
-data_module.setup()
+# Remove date column from data before passing to Dataset
+df_for_dataset = df_processed.drop(columns=['date']) if 'date' in df_processed.columns else df_processed
 
-print(f"   DataModule created successfully")
-if hasattr(data_module, 'data_processed') and data_module.data_processed is not None:
-    print(f"   Processed data shape: {data_module.data_processed.shape}")
-else:
-    print(f"   Data shape: {df_processed.shape}")
-if data_module.time_index is not None:
-    print(f"   Time range: {data_module.time_index[0]} to {data_module.time_index[-1]}")
+dataset = DDFMDataset(
+    data=df_for_dataset,  # Pass DataFrame directly (not .values) - already preprocessed
+    time_idx='index',  # Use DataFrame index as time identifier
+    target_series=[target_col],  # Specify target series
+    target_scaler=y_scaler  # Fitted scaler for target series
+)
+
+print(f"   Dataset created successfully")
+print(f"   Data shape: {dataset.data.shape}")
+print(f"   Target series: {dataset.target_series}")
 
 # ============================================================================
 # Step 5: Train Model
 # ============================================================================
 print("\n[Step 5] Training DDFM model...")
 
+# Create model with parameters from config
+# Map config parameters to model parameters
+encoder_layers = getattr(config, 'encoder_layers', [32, 1])
+encoder_size = tuple(encoder_layers) if encoder_layers else (32, 1)
+max_epoch = getattr(config, 'max_epoch', 3)  # Reduced for faster execution
+
 model = DDFM(
-    encoder_layers=[32, 16],  # Reduced for faster execution
-    num_factors=1,  # Reduced to 1 for faster execution
-    factor_order=1,  # VAR(1) - first-order factor dynamics
-    epochs=10,  # Reduced for faster execution
-    max_iter=3,  # Reduced for faster execution
-    batch_size=DEFAULT_BATCH_SIZE,  # Reduced for faster execution
-    learning_rate=DEFAULT_DDFM_LEARNING_RATE
+    dataset=dataset,
+    config=config,
+    encoder_size=encoder_size,  # Convert list to tuple
+    decoder_type="linear",
+    activation=getattr(config, 'activation', 'relu'),
+    learning_rate=getattr(config, 'learning_rate', DEFAULT_DDFM_LEARNING_RATE),
+    optimizer='Adam',
+    n_mc_samples=getattr(config, 'n_mc_samples', 10),
+    window_size=getattr(config, 'window_size', DEFAULT_DDFM_WINDOW_SIZE),
+    max_iter=max_epoch,  # Map max_epoch -> max_iter
+    tolerance=getattr(config, 'tolerance', 0.0005),
+    disp=getattr(config, 'disp', 10),
+    seed=getattr(config, 'seed', None)
 )
-model._config = config  # Set config directly
-
-trainer = DDFMTrainer(max_epochs=1)  # Minimal epochs for faster execution
-trainer.fit(model, data_module)
 
+# Train model - fit() builds model, pre-trains, and trains in one method
+print(f"   Starting training (max {model.max_iter} iterations)...")
+model.fit()
 print("   Training completed!")
 
+# Build state-space model for prediction
+print("\n[Step 5.5] Building state-space model...")
+model.build_state_space()
+print("   State-space model built successfully")
+
 # ============================================================================
 # Step 6: Prediction
 # ============================================================================
 print("\n[Step 6] Making predictions...")
 
-# Predict with horizon=6 (uses target_series from DataModule)
-X_forecast, Z_forecast = model.predict(horizon=6)
+# Predict with horizon=6 (uses target_series from Dataset)
+# Note: predict() requires build_state_space() to be called first
+X_forecast, Z_forecast = model.predict(horizon=6, return_series=True, return_factors=True)
 
 print(f"   Forecast shape: {X_forecast.shape}")
 print(f"   Factor forecast shape: {Z_forecast.shape}")
